/*
 * C
 *
 * Copyright 2012-2015 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include "buttons.h"
#include "cpu.h"
#if MCU == 2
#include "stm32f2xx.h"
#include "stm322xg_eval.h"
#elif MCU == 4
#include "stm32f4xx.h"
#include "stm324xg_eval.h"
#else
#error Invalid MCU
#endif
#include "microej.h"
#include "interrupts.h"

// this h file is created by buildSystemMicroUI step
#include "microui_constants.h"
#include "LLCOMM_UART.h"

// include LLINPUT to send a button event to MicroUI when buttons event occurs
#include "LLINPUT.h"

static GPIO_TypeDef* BUTTON_PORT[BUTTONn] = {WAKEUP_BUTTON_GPIO_PORT, TAMPER_BUTTON_GPIO_PORT, KEY_BUTTON_GPIO_PORT};
static const uint16_t BUTTON_PIN[BUTTONn] = {WAKEUP_BUTTON_PIN, TAMPER_BUTTON_PIN, KEY_BUTTON_PIN};
static const uint32_t BUTTON_CLK[BUTTONn] = {WAKEUP_BUTTON_GPIO_CLK, TAMPER_BUTTON_GPIO_CLK, KEY_BUTTON_GPIO_CLK};
static const uint16_t BUTTON_EXTI_LINE[BUTTONn] = {WAKEUP_BUTTON_EXTI_LINE, TAMPER_BUTTON_EXTI_LINE, KEY_BUTTON_EXTI_LINE};
static const uint8_t BUTTON_PORT_SOURCE[BUTTONn] = {WAKEUP_BUTTON_EXTI_PORT_SOURCE, TAMPER_BUTTON_EXTI_PORT_SOURCE, KEY_BUTTON_EXTI_PORT_SOURCE};
static const uint8_t BUTTON_PIN_SOURCE[BUTTONn] = {WAKEUP_BUTTON_EXTI_PIN_SOURCE, TAMPER_BUTTON_EXTI_PIN_SOURCE, KEY_BUTTON_EXTI_PIN_SOURCE};
static const uint8_t BUTTON_IRQn[BUTTONn] = {WAKEUP_BUTTON_EXTI_IRQn, TAMPER_BUTTON_EXTI_IRQn, KEY_BUTTON_EXTI_IRQn};

// lookup table that indicates whether a GPIO pin value of 1 indicates pressed or released
// true if 0 means "pressed" and 1 means "released"; false if 1 means "pressed" and 0 means "released"
static const uint8_t BUTTON_REVERSE[BUTTONn] = {MICROEJ_FALSE, MICROEJ_TRUE, MICROEJ_TRUE};

// table that records the last state of the button
static uint8_t BUTTON_PRESSED[BUTTONn] = {MICROEJ_FALSE, MICROEJ_FALSE, MICROEJ_FALSE};

static void BUTTONS_event(Button_TypeDef Button) {
	uint8_t pin_is_one;
	uint8_t button_pressed;

	pin_is_one = GPIO_ReadInputDataBit(BUTTON_PORT[Button], BUTTON_PIN[Button]) == Bit_SET;
	button_pressed = (pin_is_one && !BUTTON_REVERSE[Button]) || (!pin_is_one && BUTTON_REVERSE[Button]);

	if (button_pressed) {
		if (!BUTTON_PRESSED[Button])	{
			// button was previously released, so this is a press event
			if (LLINPUT_sendButtonPressedEvent(MICROUI_EVENTGEN_BUTTONS, Button)) {
				// the event has been queued: we can store the new button state
				BUTTON_PRESSED[Button] = MICROEJ_TRUE;
			} else {
				// the event has not been queued. We must not change the button state to prevent
				// sending a future release event when the press event has never been sent!
			}
		}
	} else {
		// button is released
		if (BUTTON_PRESSED[Button]) {
			// button was previously pressed, so this is a release event
			if (LLINPUT_sendButtonReleasedEvent(MICROUI_EVENTGEN_BUTTONS, Button)) {
				// the event has been queued: we can store the new button state
				BUTTON_PRESSED[Button] = MICROEJ_FALSE;
				comm_impl_event(Button);
			} else {
				// the event has not been queued. We must not change the button state to prevent
				// sending a future press event when the release event has never been sent!
			}
		}
	}
}

static void BUTTONS_interrupt(Button_TypeDef Button) {
	if(EXTI_GetITStatus(BUTTON_EXTI_LINE[Button]) != RESET) 	{
		BUTTONS_event(Button);
		EXTI_ClearITPendingBit(BUTTON_EXTI_LINE[Button]);
	}
}

static void BUTTONS_init(Button_TypeDef Button) {
	GPIO_InitTypeDef GPIO_InitStructure;
	EXTI_InitTypeDef EXTI_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	/* Enable the BUTTON Clock */
	RCC_AHB1PeriphClockCmd(BUTTON_CLK[Button], ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

	/* Configure Button pin as input */
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Pin = BUTTON_PIN[Button];
	GPIO_Init(BUTTON_PORT[Button], &GPIO_InitStructure);

	/* Connect Button EXTI Line to Button GPIO Pin */
	SYSCFG_EXTILineConfig(BUTTON_PORT_SOURCE[Button], BUTTON_PIN_SOURCE[Button]);

	/* Configure Button EXTI line */
	EXTI_InitStructure.EXTI_Line = BUTTON_EXTI_LINE[Button];
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_InitStructure);

	/* Enable and set Button EXTI Interrupt to the lowest priority */
	NVIC_InitStructure.NVIC_IRQChannel = BUTTON_IRQn[Button];
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0F;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0F;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

	NVIC_Init(&NVIC_InitStructure);
}

static void setButtonsInterrupts(FunctionalState state) {
	EXTI_InitTypeDef EXTI_InitStructure;
	EXTI_InitStructure.EXTI_Line = BUTTON_EXTI_LINE[BUTTON_WAKEUP] | BUTTON_EXTI_LINE[BUTTON_KEY] | BUTTON_EXTI_LINE[BUTTON_TAMPER];
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
	EXTI_InitStructure.EXTI_LineCmd = state;
	EXTI_Init(&EXTI_InitStructure);
}

/**
 * @brief  This function handles External line 0 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI0_IRQHandler(void) {
	enterInterrupt();
	BUTTONS_interrupt(BUTTON_WAKEUP);
	leaveInterrupt();
}

/**
 * @brief  This function handles External lines 15 to 10 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI15_10_IRQHandler(void) {
	enterInterrupt();
	BUTTONS_interrupt(BUTTON_KEY);
	BUTTONS_interrupt(BUTTON_TAMPER);
	leaveInterrupt();
}

/**
 * @brief  Enable the buttons interrupts if the number of times disableButtonsInterrupts
 * has been called is exactly one greater than the number of times enableButtonsInterrupts
 * has been called.
 * @param  None
 * @retval None
 */
void enableButtonsInterrupts(void) {
	setButtonsInterrupts(ENABLE);
}

/**
 * @brief  Disable the buttons interrupts.
 * @param  None
 * @retval None
 */
void disableButtonsInterrupts(void) {
	setButtonsInterrupts(DISABLE);
}

void buttonsInitialize(void) {
	BUTTONS_init(BUTTON_WAKEUP);
	BUTTONS_init(BUTTON_TAMPER);
	BUTTONS_init(BUTTON_KEY);
	enableButtonsInterrupts();
}
